/* Driver program for lzrw*.c by Ross Williams
Written by Matt Mahoney, Feb. 14, 2008.
This program is released into the public domain.

To compile (g++ 3.4.2)
gcc -O2 -fomit-frame-pointer -march=pentiumpro %1.c main.c -o %1
upx -qqq %1.exe

where %1 is one of lzrw1-a, lzrw2, lzrw3, lzrw3-a, lzrw5
from http://www.ross.net/compression/

To compress:   %1 c input output
To decompress: %1 d input output

The driver program divides the input into 1 MB block and
compresses each with a 4 byte header giving the compressed size
as a big-endian number.
*/

#include <stdio.h>
#include <stdlib.h>
#include "port.h"
#include "fast_copy.h"
#include "compress.h"
#define BUFSIZE 0x100000  /* change to 0x10000 for lzrw5.c */

void fast_copy(void *src, void *dest, unsigned long len) {
  memcpy(dest, src, len);
}

int main(int argc, char** argv) {
  FILE *in, *out;  /* input, output files */
  static UBYTE  inb[BUFSIZE+COMPRESS_OVERRUN]; 
  static UBYTE outb[BUFSIZE+COMPRESS_OVERRUN];  /* input, output buffers */
  ULONG inlen=0, outlen=0;  /* input, output lengths */
  struct compress_identity* ci;  /* algorithm info, see compress.h */
  UBYTE *work;  /* working memory */

  /* Check args */
  if (argc<4 || argv[1][0]!='c' && argv[1][0]!='d') {
    fprintf(stderr, "To compress:   %s c input output\n"
                    "To decompress: %s d input output\n", argv[0], argv[0]);
    exit(1);
  }

  /* Get working memory */
  compress(COMPRESS_ACTION_IDENTITY, 0, 0, 0, 0, (ULONG*)&ci);
  work=(UBYTE*)calloc(1, ci->memory);
  if (!work) printf("Out of memory\n"), exit(1);

  /* Open files */
  in=fopen(argv[2], "rb");
  if (!in) perror(argv[2]), exit(1);
  out=fopen(argv[3], "wb");
  if (!out) perror(argv[3]), exit(1);

  /* Compress */
  if (argv[1][0]=='c') {
    while ((inlen=fread(inb, 1, BUFSIZE, in))>0) {
      compress(COMPRESS_ACTION_COMPRESS, work, inb, inlen, outb, &outlen);
      fprintf(out, "%c%c%c%c", outlen>>24, outlen>>16, outlen>>8, outlen);
      fwrite(outb, 1, outlen, out);
    }
  }

  /* Decompress */
  else {
    int c;
    while ((c=getc(in))!=EOF) {
      inlen=c<<24;
      inlen|=getc(in)<<16;
      inlen|=getc(in)<<8;
      inlen|=getc(in);
      fread(inb, 1, inlen, in);
      compress(COMPRESS_ACTION_DECOMPRESS, work, inb, inlen, outb, &outlen);
      fwrite(outb, 1, outlen, out);
    }
  }
  return 0;
}
